#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from zope.schema.interfaces import ITextLine, IChoice, ISet

# import packages
from zope.interface import implementer, Interface
from zope.schema import TextLine, Choice, Set


class IPermissionField(IChoice):
    """Permission field interface"""


@implementer(IPermissionField)
class PermissionField(Choice):
    """Permission field"""

    def __init__(self, **kwargs):
        if 'vocabulary' in kwargs:
            del kwargs['vocabulary']
        super(PermissionField, self).__init__(vocabulary='PyAMS permissions', **kwargs)


class IPermissionsSetField(ISet):
    """Permissions set field interface"""


@implementer(IPermissionsSetField)
class PermissionsSetField(Set):
    """Permissions set field"""

    def __init__(self, **kwargs):
        if 'value_type'in kwargs:
            del kwargs['value_type']
        super(PermissionsSetField, self).__init__(value_type=PermissionField(), **kwargs)


class IRoleField(Interface):
    """Base role field interface"""

    role_id = TextLine(title="Matching role ID",
                       required=False)


class IPrincipal(ITextLine, IRoleField):
    """Principal field interface"""


@implementer(IPrincipal)
class Principal(TextLine):
    """Principal field"""

    role_id = None

    def __init__(self, **kwargs):
        if 'role_id' in kwargs:
            self.role_id = kwargs.pop('role_id')
        super(Principal, self).__init__(**kwargs)


class IPrincipalsSet(ISet, IRoleField):
    """Principals set interface"""


@implementer(IPrincipalsSet)
class PrincipalsSet(Set):
    """Principals set field"""

    role_id = None

    def __init__(self, **kwargs):
        if 'role_id' in kwargs:
            self.role_id = kwargs.pop('role_id')
        super(PrincipalsSet, self).__init__(**kwargs)
